clc
clear
%% I. Specify the operating situation
shape= 'rectangular';  % The options include 'rectangular', 'triangular', and 'semicircular'.
AR=2;      % Specify channel aspect ratio
Re=200;     % Specify channel Reynolds number
kappa=0.2; % Specify particle blockage ratio

k=0;
if strcmp(shape, 'rectangular')
    lateral_y =0:0.1:(0.9-kappa);   % Specify corresponding 2y/H
    lateral_z =0:0.1:(AR-0.1-kappa); % Specify corresponding 2z/H
    for i=1:1:length(lateral_y)
        for j=1:1:length(lateral_z)
            k=k+1;
            Output(:,k)=[AR;Re;kappa;lateral_y(i);lateral_z(j)./AR];
        end
    end
elseif strcmp(shape, 'semicircular')
    for i=1:1:20                 % Specify corresponding 2y/H
        for j=1:1:20            % Specify corresponding 2z/H
            lateral_y=i*0.1;
            lateral_z=(j-1)*0.1;
            if lateral_y^2+lateral_z^2<(2-kappa-0.099)^2 && lateral_y>(kappa+0.099)
                k=k+1;
                Output(:,k)=[Re;kappa;lateral_y-1;lateral_z];
            end
        end
    end
elseif strcmp(shape, 'triangular')
    num=floor(19-kappa/0.05);
    low=-(0.84-(kappa-0.1)/0.05*0.04); % The maximum value of 2y/H
    up=(0.44-(kappa-0.1)/0.05*0.04);   % The minimum value of 2y/H
    for i=1:1:num
        for j=1:1:num+1-i
            lateral_y=(i-1)*(up-low)/(num-1)+low;           % Specify corresponding 2y/H
            lateral_z=(j-1)*(up-lateral_y)/2/(num-i);       % Specify corresponding 2z/H
            if i>num-1
                lateral_z=0;
            end
            k=k+1;
            Output(:,k)=[Re;kappa;lateral_y;lateral_z];
        end
    end
else error('Please choose the channel shape');
end

%% II. Import the database
database = importdata(['lift_database_',shape,'.data'])';
if strcmp(shape, 'rectangular')
    Input =database(1:5,:);             % Five elements: AR Re kappa 2y/H 2z/H
    Input(5,:) =Input(5,:)./Input(1,:); % Preprocessing of the 2z/H
    Target_CL =database(6:7,:);         % Two elements: CLy and CLz
else
    Input =database(1:4,:);             % Four elements: Re kappa 2y/H 2z/H
    Target_CL =database(5:6,:);         % Two elements: CLy and CLz
end
%% III. 0-1 normalization
[input, ps_input] = mapminmax(Input,0,1);
output = mapminmax('apply',Output,ps_input);
[target_CL, ps_CL] = mapminmax(Target_CL,0,1);

%% IV. Neural Network Training
% 1. Build the network
net_CL = newff(input,target_CL,[20 8],{'tansig' 'tansig' 'purelin'});

% 2. Set training parameters
net_CL.trainParam.epochs = 1000;
net_CL.trainParam.goal = 1e-5;
net_CL.trainParam.mu = 0.001;

% 3. Perform the network
net_CL = train(net_CL,input,target_CL);

%% V Neural Network Prediction
predict_CL = sim(net_CL,output);

%% VI Anti-normalization
Predict_CL = mapminmax('reverse',predict_CL,ps_CL);

%% VII. Output lift distribution for the operating situation
fp = fopen('lift.data','w+');
fprintf(fp, '%s\n',[shape,' channel']);
fprintf(fp, '%s\t','VARIABLES="2y/H","2z/H","CLy","CLz"');
fprintf(fp, '\n');
if strcmp(shape, 'rectangular')
    fprintf(fp, '%s\t',['operating situation: ','AR=',num2str(AR),', Re=',num2str(Re),', kappa=',num2str(kappa,'%.2f')]);
    for i=1:length(Output(1,:))
        fprintf(fp, '\n');
        fprintf(fp, '%5f\t',Output(4,i), Output(5,i)*AR,Predict_CL(1,i),Predict_CL(2,i));
    end
else
    fprintf(fp, '%s\t',['operating situation: ','Re=',num2str(Re),', kappa=',num2str(kappa,'%.2f')]);
    for i=1:length(Output(1,:))
        fprintf(fp, '\n');
        fprintf(fp, '%5f\t',Output(3,i), Output(4,i),Predict_CL(1,i),Predict_CL(2,i));
    end
end
fclose(fp);
